
// demo-is-self-contained

#include "comb/comb-print.h"

#include "fxttypes.h"
#include "fxtiomanip.h"
#include "jjassert.h"
#include "nextarg.h"


//% Recursive generation of mixed radix numbers in a minimal-change order
//% related so subset-lex order ("SL-Gray" order).


// Cf. mixedradix-sl-gray-demo.cc for an iterative generator.


ulong N;
ulong K;
ulong *rv;  // digits of mixed radix number

ulong ct;   // count objects


void visit()
{
    ++ct;
    cout << setw(4) << ct << ":";
    print_vec("  ", rv, N, 1);
    cout << endl;
}
// -------------------------

void
slg_rec(ulong n, bool z=true)
{
    if ( n==N )
    {
        visit();
        return;
    }

    const ulong k = K;  // fixed radix
//    const ulong k = n + 2;  // rising factorial basis
//    const ulong k = N - n + 1;  // falling factorial basis

//    z = 1;  // generate subset-lex order
    if ( z )  // forward:
    {
        for (ulong j=1; j<k; ++j)  // rising nonzero values followed by zeros
        {
            rv[n] = j;
            for (ulong i=n+1; i<N; ++i)  rv[i] = 0;
            visit();
        }

        if ( n < N - 1 )  // only if not at end
        {
            for (ulong j=k-1; (long)j>=0; --j)  // falling values and recursion
            {
                rv[n] = j;
                slg_rec(n+1, z);
                z = !z;
            }
        }
    }
    else     // backward:
    {
        if ( n < N - 1 )  // only if not at end
        {
            z = k & 1;  // fixup for odd k: nine followed by forward list

            for (ulong j=0; j<k; ++j)  // rising values and recursion
            {
                z = !z;
                rv[n] = j;
                slg_rec(n+1, z);
            }
        }

        for(ulong j=k-1; j>0; --j)  // falling nonzero values followed by zeros
        {
            rv[n] = j;
            for (ulong i=n+1; i<N; ++i)  rv[i] = 0;
            visit();
        }
    }
}
// -------------------------

int
main(int argc, char **argv)
{
    N = 3;
    NXARG(N, "Number of digits");
    K = 4;
    NXARG(K, "Radix");

    rv = new ulong[N];
    for (ulong j=0; j<N; ++j)  rv[j] = 0;

    ct = 0;
    visit();  // note
    slg_rec(0, true);

    cout << "ct=" << ct << endl;

    delete [] rv;

    return 0;
}
// -------------------------


/// Emacs:
/// Local Variables:
/// MyRelDir: "demo/comb"
/// makefile-dir: "../../"
/// make-target: "1demo DSRC=demo/comb/mixedradix-sl-gray-rec-demo.cc"
/// make-target2: "1demo DSRC=demo/comb/mixedradix-sl-gray-rec-demo.cc DEMOFLAGS=-DTIMING"
/// End:
